%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: rossler_pair_ode.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    The ordinary differential equations characterising the Rossler
%    oscillator.  The function relies on externally set parameters
%    w1 w2 eps1 eps2 a1 a2.  Variable mapping is 
%
%    x1,y1,z1 = x(1), x(2), x(3) and
%    x2,y2,z2 = x(4), x(5), x(6).
%
% Arguments:
%
%    w1, w2 [global] : the frequency of each oscillator
%
%    eps1, eps2 [global] : the coupling strength 1->2 (eps2) and 2->1 (eps1)
%
%    a1, a2 [global]: the a parameter in the Rossler systems
%
%    x : the current state of the system (6x1 matrix)
%
%    t : the current time (unused)
%
% Returns:
%
%    xdot : a (6x1 matrix) of derivatives of the state variables
%
%
function xdot = rossler_pair_ode(t, x)
	
	global eps1 eps2 w1 w2 a1 a2
	
	xdot = zeros(6,1);
	
	xdot(1) = - w1 * x(2) - x(3) + eps1 * (x(4) - x(1));
	xdot(2) = w1 * x(1) + a1 * x(2);
	xdot(3) = 0.2 + x(3) * (x(1) - 10);
	
	xdot(4) = - w2 * x(5) - x(6) + eps2 * (x(1) - x(4));
	xdot(5) = w2 * x(4) + a2 * x(5);
	xdot(6) = 0.2 + x(6) * (x(4) - 10);
	
end
