%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: van_der_pol_pair_heun.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    A function integrating the stochastic Van der Pol system
%    using the predictor corrector Heun scheme.  The function 
%    relies on externally set parameters w1 w2 eps1 eps2 mu.
%    Variable mapping is 
%
%	x1,y1 = x(1), x(2) and
%	x2,y2 = x(3), x(4).
%
%    Using this function will provide the same result as
%    using the sde_heun_solver function with the van_der_pol_pair_ode
%    ODE but is much faster.
%
%
% Synopsis:
%
%    [t, x] = xdot = van_der_pol_pair_heun(x0, td, Nreq, subsampling)
%
% Arguments:
%
%    mu [global] : the 'nonlinearity parameter'
%
%    eps1, eps2 [global] : the coupling strength 1->2 (eps2) and 2->1 (eps1)
%
%    w1, w2 [global]: frequencies of the oscillators
%
%    x0 : the initial state of the system, row vector
%
%    dt : the integration time step
%
%    xi : the vector of standard deviations of the inserted gaussian noise
%        (variable mapping same as for x0 and x), 0 means no inserted noise, row
%        vector
%
%    Nreq : the requested no. of points (after subsampling)
%
%    subsampling : subsampling rate (subsampling is done internally)
%
%
% Returns:
%
%
%    t : a column vector of time instants when the state is available
%
%    x : the state matrix (one column for each variable, row corresponds to
%        time instant)
%
%
function [t, x] = van_der_pol_pair_heun(x0, xi, dt, Nreq, subsmp, mu, w1, w2, eps1, eps2)
	
    sdt = sqrt(dt);

    % results
    t = zeros(Nreq, 1);
    x = zeros(Nreq, 4);
    x(1,:) = x0;
    
    % prepare state vars
    xdot = zeros(1, 4);
    xdot2 = zeros(1, 4);
    xt = x0;
    tt = 0;
    
    ss = subsmp;
    ndx = 2;
    
    for i=2:Nreq*subsmp
        
        % draw random value
        dW = randn(1,4) .* xi * sdt;
        
        % perform prediction
        xdot(1) = mu * (1 - xt(2)^2) * xt(1) - w1 * w1 * xt(2) + eps1 * (xt(4) - xt(2));
        xdot(2) = xt(1);
        xdot(3) = mu * (1 - xt(4)^2) * xt(3) - w2 * w2 * xt(4) + eps2 * (xt(2) - xt(4));
        xdot(4) = xt(3);
        
        xprime = xt + xdot * dt + dW;
        
        % new derivative step
        xdot2(1) = mu * (1 - xprime(2)^2) * xprime(1) - w1 * w1 * xprime(2) + eps1 * (xprime(4) - xprime(2));
        xdot2(2) = xprime(1);
        xdot2(3) = mu * (1 - xprime(4)^2) * xprime(3) - w2 * w2 * xprime(4) + eps2 * (xprime(2) - xprime(4));
        xdot2(4) = xprime(3);
        
        % save resulting value
        xt = xt + 0.5 * (xdot + xdot2) * dt + dW;
        tt = tt + dt;
        
        % perform subsampling
        ss = ss - 1;
        if ss == 0
            x(ndx, :) = xt;
            t(ndx) = tt;
            ss = subsmp;
            ndx = ndx + 1;
        end
        
    end
	
end
