%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: van_der_pol_pair_ode.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    The ordinary differential equations characterising the Van der Pol
%    oscillator.  The function relies on externally set parameters
%    w1 w2 eps1 eps2 mu.  Variable mapping is 
%
%    x1,y1 = x(1), x(2) and
%    x2,y2 = x(3), x(4).
%
% Synopsis:
%
%    xdot = van_der_pol_pair_ode(t, x)
%
% Arguments:
%
%    mu [global] : the 'nonlinearity parameter'
%
%    eps1, eps2 [global] : the coupling strength 1->2 (eps2) and 2->1 (eps1)
%
%    w1, w2 [global]: frequencies of the oscillators
%
%    x : the current state of the system (4x1 matrix)
%
%    t : the current time (unused, system is autonomous)
%
% Returns:
%
%    xdot : a (4x1 matrix) of derivatives of the state variables
%
%
function xdot = van_der_pol_pair_ode(t, x)
	
	global eps1 eps2 w1 w2 mu
	
	xdot = zeros(4, 1);
	
	xdot(1) = mu * (1 - x(2)^2) * x(1) - w1 * w1 * x(2) + eps1 * (x(4) - x(2));
	xdot(2) = x(1);
	
	xdot(3) = mu * (1 - x(4)^2) * x(3) - w2 * w2 * x(4) + eps2 * (x(2) - x(4));
	xdot(4) = x(3);
	
end
