%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: estimate_mutual_inf.m
%    Version: 1.1
%	 Date: 15 May 2011
%    License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
% 	Estimate the mutual information between all time series in the matrix
%   columns using the ranking method.  Please see help of
%   'compute_partition' for specifics.
%
% Synopsis:
%
%	mi = estimate_mutual_info(wph, bins)
%
% Arguments:
%
%   ts : the matrix of time series, one time series per column
%
%   bins : the number of bins into which each series is to be split
%
%   strategy : either 'equidist' or 'equiquant' or 'perm'
%
% Returns:
%
%   mi : the mutual information of the time series
%
% Examples:
%
%	None.
%
% References:
%
% M Palus, “Testing for nonlinearity using redundancies: Quantitative and qualitative aspects”,
%          Physica D, 80, 186–205, (1995)
% R Moddemeijer, “On Estimation of Entropy and Mutual Information of Continuous Distributions”,
%          Signal Processing, vol. 16(3), 233–246 (1989)
%
%
function mi = estimate_mutual_inf(ts, bins, strategy)

	% length of input time series
	[M, N] = size(ts);

	% compute the binning for all time series at once
    bins_ts = zeros(bins, N);
    
    % computes the partition of all time series
    ndx_ts = compute_partition(ts, bins, strategy);
    
    % accumulate 1D bin frequencies for each time series, normalize to
    % probability estimates and compute shannon entropy
    for i=1:N
        bins_ts(:, i) = accumarray(ndx_ts(:, i), ones(1, size(ndx_ts,1)), [bins, 1]);
    end
    
    bins_ts = bins_ts / M;
    H_ts = shannon_entropy(bins_ts);

    % compute the mutual information matrix between i, j time series
    mi = zeros(N);
    mi = mi + diag(ones(1,N) * log(bins));
    for i=1:N
        ndx_xy_base = (ndx_ts(:, i) - 1) * bins;

        for j=i+1:N

            ndx_xy = ndx_xy_base + ndx_ts(:, j);
            bins_xy = accumarray(ndx_xy, ones(1, size(ndx_ts,1)), [bins*bins,1]);
   
            % normalize bin frequencies to probabilities, compute mi and
            % assign to proper places in the matrix
            bins_xy = bins_xy / M;
            mi_ij = H_ts(i) + H_ts(j) - shannon_entropy(bins_xy);
            mi(j,i) = mi_ij;
            mi(i,j) = mi_ij;
            
        end
    end
