%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008-2011 Martin Vejmelka
% 
%	 File: estimate_equiquant_redundancy.m
%    Version: 1.1
%	 Date: 15 May 2011
%    License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
% 	Estimate the redundancy between the given samples.  This is the
% 	information I(X_1; X_2; ...; X_M) = sum_i H(X_i) - H(X_1, X_2, ..., X_M)
%
% Synopsis:
%
%	mi = estimate_equiquant_redundancy(sample, bins)
%
% Arguments:
%
%   sample : the matrix of samples (each variable in one column)
%
%   bins : the number of bins into which each series is to be split
%
% Returns:
%
%   mi : the redundancy of the time series
%
%   H_sum : the sum of all 1D entropies 
%
%   H_all : the joint entropy of all time series
%
% Examples:
%
%	None.
%
function [mi, H_sum, H_all] = estimate_redundancy(sample, bins, strategy)

    % number of samples
	[M, N] = size(sample);
    
    % find the equiquantal partition
    assgn = compute_partition(sample, bins, strategy);
    
    % allocate marginal bins
    bins_hi = zeros(bins, N);
    
    % alloc marginal bins in vectorized form
    for i=1:N
        bins_hi(:, i) = accumarray(assgn(:, i), ones(1, size(assgn,1)), [bins,1]);
    end
    
    % alloc multidimensional bins
    map_radix = bins .^ (N-1:-1:0);
    bin_pos = sum((assgn-1) .* repmat(map_radix, size(assgn,1), 1), 2) + 1;
    bins_all = accumarray(bin_pos, ones(1, size(assgn,1)), [bins^N, 1]);
    
    % normalize frequencies to "probabilities"
    bins_hi = bins_hi / M;
    bins_all = bins_all / M;

    % compute redundancy
    H_sum = sum(shannon_entropy(bins_hi));
    H_all = shannon_entropy(bins_all);
    mi = H_sum - H_all;
    
    