%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: estimate_eqq_cmi.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    Estimates the conditional mutual information (CMI) using
%    the equiquantal method for a given range of time lags. The
%    method works with phase time series as input.
%
% Synopsis:
%
%    cmi = estimate_eqq_cmi(ts, win_size, bin_count, lags)
%
% Arguments:
%
%    ts : two input time series, each time series is one column.
%
%    win_size : the size of the window for CMI estimation
%
%    bin_count : the number of bin_count to use to estimate the CMI
%
%    lags : the range of lags for which CMI is to be computed
%
%    shift_op_id : the shift operator to be applied to the time series, must be a string
%                  identifier. for available shift functions, see
%                  'utility/lookup_shift_function.m'.
%
% Returns:
%
%    cmi: the conditional mutual information estimates for each lag
%         the first column contains the estimates for CMI 1->2 (lagged)
%         and the second column contains the estimate CMI 2->1 (lagged)
%	
%
function cmi = estimate_eqq_cmi(ts, win_size, bin_count, lags, shift_op_id)

	% PRE1: check if there is aenough samples in the range
	if(size(ts,1) < win_size + max(lags))
		error('Not enough samples in time series');
	end

	if(size(ts,2) < 2)
		error('Function requires two columns to work.');
	end

	if(bin_count < 2)
		error('At least two bin_count are required to work with the estimator.');
	end

	% retrieve a handle to the shift operation function
	shift_op = lookup_shift_function(shift_op_id);

	if(max(ts(:)) >= 2*pi && ~isempty(strfind(shift_op_id, 'phase')))
		error('Shift operation id indicates phase time series, but signal is not in range (0-2pi).');
	end

	% compute partitions of fixed series (X, and Y), not dependent on lags
	part_xy = eqq_partition(ts(1:win_size, :), bin_count);

	% preallocate buffers used for entropies not dependent on lags: H(X,Y), H(X), H(Y)
	buf_x = zeros(bin_count, 1);
	buf_y = zeros(bin_count, 1);
	buf_xy = zeros(bin_count * bin_count, 1);

	% accumulate the bin buffers for X and Y & normalize
	for i=1:win_size

		% 1D probability mass functions
		buf_x(part_xy(i,1)) = buf_x(part_xy(i,1)) + 1;
		buf_y(part_xy(i,2)) = buf_y(part_xy(i,2)) + 1;

		% 2D probability mass functions
		bin = (part_xy(i,1) - 1) * bin_count + part_xy(i,2);
		buf_xy(bin) = buf_xy(bin) + 1;

	end
	buf_x = buf_x / win_size;
	buf_y = buf_y / win_size;
	buf_xy = buf_xy / win_size;

	% H(X), H(Y) and H(X,Y) is not affected by shifting - precompute now
	hxy = shannon_entropy(buf_xy);
	hx = shannon_entropy(buf_x);
	hy = shannon_entropy(buf_y);

	% result buffer for lagged CMI
	cmi = zeros(size(lags,1), 2);

	% direction 1->2: for each phase shift estimate the CMI and copy to cmi(:,1)
	for i=1:length(lags)

		% perform lagging for both series at the same time
		lag = lags(i);
		shifted_series = shift_op(ts, win_size, lag);

		% compute equiquantal partition
		part_shifted = eqq_partition(shifted_series, bin_count);
		
		% prepare buffers for entropy computations: H(X,X_T), H(Y,Y_T), H(Y,X,X_T), H(X,Y,Y_T)
		buf2x = zeros(bin_count * bin_count, 1);
		buf2y = zeros(bin_count * bin_count, 1);
		buf3x = zeros(bin_count * bin_count * bin_count, 1);
  		buf3y = zeros(bin_count * bin_count * bin_count, 1);

		% accumulate lag dependent bin_count: H(Y,Y_T) and H(X,Y,Y_T)
		for k=1:win_size

			% bin_count for H(X,X_T)
			ndx2 = (part_shifted(k, 1) - 1) * bin_count + part_xy(k, 1);
			buf2x(ndx2) = buf2x(ndx2) + 1;

			% bin_count for H(Y,Y_T)
			ndx2 = (part_shifted(k, 2) - 1) * bin_count + part_xy(k, 2);
			buf2y(ndx2) = buf2y(ndx2) + 1;

			% bin_count for H(X,Y,Y_T)
			ndx3 = ((part_shifted(k, 2) - 1) * bin_count + part_xy(k, 2) - 1) * bin_count + part_xy(k, 1);
			buf3y(ndx3) = buf3y(ndx3) + 1;

			% bin_count for H(Y,X,X_T)
			ndx3 = ((part_shifted(k, 1) - 1) * bin_count + part_xy(k, 1) - 1) * bin_count + part_xy(k, 2);
			buf3x(ndx3) = buf3x(ndx3) + 1;
		end

		% normalize buffers
		buf2x = buf2x / win_size;
		buf2y = buf2y / win_size;
		buf3x = buf3x / win_size;
		buf3y = buf3y / win_size;

		% compute lag-dependent entropies
		hxxt = shannon_entropy(buf2x);
		hyyt = shannon_entropy(buf2y);
		hyxxt = shannon_entropy(buf3x);
		hxyyt = shannon_entropy(buf3y);

		% compute the CMI for both directions at once
		cmi(i,1) =  hxy + hyyt - hy - hxyyt;
		cmi(i,2) =  hxy + hxxt - hx - hyxxt;

	end

end
