%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: estimate_perm_cmi.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    Estimates the conditional mutual information (CMI) using
%    the permutation entropy method (3 neighboring points used
%    for comparison) for a given range of time  lags. The method
%    works with either raw time series or phase time series.
%
% Synopsis:
%
%    cmi = estimate_perm_cmi(ts, win_size, npts, lags, shift_op_id)
%
% Arguments:
%
%    ts : two input time series, each time series is one column.
%
%    win_size : the size of the window for CMI estimation
%
%    npts : number of points to be used in the comparison (maximum 6)
%
%    lags : the range of lags for which CMI is to be computed
%
%    shift_op_id : the shift operator to be applied to the time series, must be a string
%                  identifier. for available shift functions, see
%                  'utility/lookup_shift_function.m'.
%
% Returns:
%
%    cmi : the conditional mutual information estimates for each lag
%          the first column contains the estimates for CMI 1->2 (lagged)
%          and the second column contains the estimate CMI 2->1 (lagged)
%
%
function cmi = estimate_perm_cmi(ts, win_size, npts, lags, shift_op_id)

	% PRE: check inputs
	if(size(ts,1) < win_size + max(lags))
		error('Not enough samples in time series');
	end

	if(size(ts,2) < 2)
		error('Function requires two columns to work.');
	end

	if(npts < 2 || npts > 6)
		error('Number of comparison points (npts) is required to be in the range 2..6');
	end

	% retrieve a handle to the shift operation function
	shift_op = lookup_shift_function(shift_op_id);

	if(max(ts(:)) >= 2*pi && ~isempty(strfind(shift_op_id, 'phase')))
		error('Shift operation id indicates phase time series, but signal is not in range (0-2pi).');
	end

	%  bin count is factorial of point counts
	bin_count = factorial(npts);

	% compute partitions of fixed series (X, and Y), not dependent on lags
	part_xy = perm_partitionN(ts(1:win_size,:), npts);

	% if the binning procedure discards some points, this is noticed here
	bin_N = size(part_xy, 1);

	% preallocate buffers used for entropies not dependent on lags: H(X,Y), H(X), H(Y)
	buf_x = zeros(bin_count, 1);
	buf_y = zeros(bin_count, 1);
	buf_xy = zeros(bin_count * bin_count, 1);

	% accumulate the bin buffers for X and Y & normalize
    contr = 1.0 / bin_N;
	for i=1:bin_N

		% 1D probability mass functions
		buf_x(part_xy(i, 1)) = buf_x(part_xy(i, 1)) + contr;
		buf_y(part_xy(i, 2)) = buf_y(part_xy(i, 2)) + contr;

		% 2D probability mass function
		bin = (part_xy(i, 1) - 1) * bin_count + part_xy(i, 2);
		buf_xy(bin) = buf_xy(bin) + contr;

	end

	% H(X), H(Y) and H(X,Y) is not affected by shifting - precompute now
	hx = shannon_entropy(buf_x);
	hy = shannon_entropy(buf_y);
	hxy = shannon_entropy(buf_xy);

	% result buffer for lagged CMI
	cmi = zeros(size(lags,1), 2);

	% accumulate both directionalities at the same time
	for i=1:length(lags)

		% perform lagging for both series at the same time
		lag = lags(i);
		shifted_series = shift_op(ts, win_size, lag);

		% compute partition with permutations
		part_shifted = perm_partitionN(shifted_series, npts);
		
		% prepare buffers for entropy computations: H(X,X_T), H(Y,Y_T), H(Y,X,X_T), H(X,Y,Y_T)
		buf2x = zeros(bin_count * bin_count, 1);
		buf2y = zeros(bin_count * bin_count, 1);
		buf3x = zeros(bin_count * bin_count * bin_count, 1);
  		buf3y = zeros(bin_count * bin_count * bin_count, 1);

		% accumulate lag dependent bins: H(Y,Y_T) and H(X,Y,Y_T)
		for k=1:bin_N

			% bins for H(X,X_T)
			ndx2 = (part_shifted(k, 1) - 1) * bin_count + part_xy(k, 1);
			buf2x(ndx2) = buf2x(ndx2) + contr;

			% bins for H(Y,Y_T)
			ndx2 = (part_shifted(k, 2) - 1) * bin_count + part_xy(k, 2);
			buf2y(ndx2) = buf2y(ndx2) + contr;

			% bins for H(X,Y,Y_T)
			ndx3 = ((part_shifted(k, 2) - 1) * bin_count + part_xy(k, 2) - 1) * bin_count + part_xy(k, 1);
			buf3y(ndx3) = buf3y(ndx3) + contr;

			% bins for H(Y,X,X_T)
			ndx3 = ((part_shifted(k, 1) - 1) * bin_count + part_xy(k, 1) - 1) * bin_count + part_xy(k, 2);
			buf3x(ndx3) = buf3x(ndx3) + contr;
		end

		% compute lag-dependent entropies
		hxxt = shannon_entropy(buf2x);
		hyyt = shannon_entropy(buf2y);
		hyxxt = shannon_entropy(buf3x);
		hxyyt = shannon_entropy(buf3y);

		% compute the CMI for both directions at once
		cmi(i,1) =  hxy + hyyt - hy - hxyyt;
		cmi(i,2) =  hxy + hxxt - hx - hyxxt;

	end

end
