%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: extract_hilbert_phase_conv.m
%     Version: 1.0;
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    This function uses the hilbert transform to compute the imaginary
%    part of a real signal and extracts a phase signal as per the analytical
%    function approach of Gabor.  Uses the convolution approach to work in
%    the time domain: convolves the impulse response of a hilbert phase shift
%    filter with the time series.  The input time series are normalized
%    internally.
%
%
% Arguments:
%
%    x: the input time series, may be a matrix, each column is one series
%
%    win_size: length of the impulse response in samples (half-window length, default = 50)
%
%
% Note:
%
%    1. This function is not optimized.
%	
%    2. Please use extract_fft_hilbert_phase.m if at all possible.
%
%    3. The phase signal will be time shifted w.r.t. the original signal by win_size samples.
%
% Returns:
%
%    wph: a wrapped phase signal for each input series
%
% Examples:
%
%    t = 1:0.1:100;
%    x = sin(t)';
%    ph = extract_fft_hilbert_phase_conv(x, 128);
%    plot(x, '-', ph, '+')
%
%
function ph = extract_hilbert_phase_conv(x, win_size)

	% default values
	if nargin < 2
		win_size = 50;
	end

	% setup
	[N, M] = size(x);
	
	% demean and normalize RMS
	mn = mean(x);
	x = x - mn(ones(N, 1), :);
	rms = std(x, 1);
	x = x ./ rms(ones(N,1),:);

	% convolve the signal with the filter
	ph = zeros(N - win_size * 2, M);
	cplx2 = zeros(N - win_size * 2, M);
	for i=win_size+1:N-win_size
	
		cplx = zeros(1,M);
		for j = 1:win_size
			cplx = cplx + (x(i-j,:) - x(i+j,:)) / j;
		end

		cplx2(i-win_size,:) = cplx / pi;
		
	end
	
	ph = atan2(cplx2, x(win_size+1:N-win_size,:)) + pi;

