%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: extract_marked_events_phase.m
%     Version: 1.0;
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    This function extracts the marked events phase by first detecting
%    the selected event in the source time series and then interpolating
%    the phase between events linearly.  If more time series are sent as
%    input, the resulting series are realigned so that the samples of
%    the phase time series are temporally aligned.  Thus the resulting
%    phase time series begin at the latest first detected event across
%    source time series.
%
%
% Synopsis:
%
%    ev_times = mark_events(x, events)
%
%
% Arguments:
%
%    x : the input time series, may be a matrix, each column is one series
%
%    events: one of the following: 'peaks', 'crossings', 'smart-crossing'
%	     please see 'mark_events.m' for an explanation of how each method
%	     works.
%
%
% Note:
%
%
%    It is advisable to plot the result of the marked events phase extraction to make
%    sure the results are useable for further analysis.
%
%
% Returns:
%
%    ph: a phase time series for each input series
%
%    ofs: the offset (in samples) of the beginning of the phase time series
%        from the first sample of the original time series
%
%
function [ph, ofs] = extract_marked_events_phase(x, event)

    % PRE: check inputs
    if nargin < 2
        event = 'crossings';
    end

    [N, M] = size(x);

    % resulting matrix is at most the same size as input matrix
    ph = zeros(N, M);

    % storage for time of first event detections
    first_events = zeros(1, M);
    last_events = zeros(1, M);

    % Process each signal separately first
    for i=1:M

        % obtain the event times
        ev_times = mark_events(x(:,i), event);

        % interpolate the time series
        ph(1:ev_times(end), i) = interpolate_phase(ev_times);

        % record when the first event and last event was detected in each series
        first_events(i) = ev_times(1);
        last_events(i) = ev_times(end);

    end

    % find the latest first event
    ev_first = max(first_events);
    ev_last = min(last_events);

    % remove part of the phase before the latest first event and after the earliest last
    % detected event
    ph = ph(ev_first:ev_last, :);

    ofs = ev_first;
