%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: interpolate_phase.m
%    Version: 1.0;
%	 Date: 14 April 2008
%    License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Synopsis:
%
%    wph = interpolate_phase(ev_times)
%
% Description:
%
%    Linearly interpolates phase between successive event times.
%
% Arguments:
%
%    ev_times: the indices of times when the marking event has occurred.
%              a column vector
%
%
% Returns:
%
%    wph: linearly interpolated wrapped phase series
%
%
% Example:
%
%   % generates a sawtooth wave
%   ev_times = 1:10:100;
%   wph = interpolate_phase(ev_times);
%   plot(wph)
%
%
function wph = interpolate_phase(ev_times)

	% init the result matrix
	wph = zeros(ev_times(end), 1); 

	% comment: phase will be zero before first event time

	% series always starts from phase value of 0
	start = ev_times(1);
	j = ev_times(1);
	for i=1:length(ev_times)-1

		% the phase increment per sample in this period
		incr = 2 * pi / (ev_times(i+1) - ev_times(i));

		% fill out the phase for this period
		while(j < ev_times(i+1)), wph(j) = incr * (j - start); j = j + 1; end

		% update the start var
		start = ev_times(i+1);

	end
