%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: mark_events.m
%     Version: 1.0;
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    This function extracts marked events from the given time series and
%    returns them in a vector for further processing.
%
% Synopsis:
%
%    ev_times = mark_events(x, events)
%
%
% Arguments:
%
%    x: the input series must be a vector
%
%    events : one of the following:
%            'peaks' - extracts successive maxima
%	     'crossings' - extracts times of positive zero crossings (default, if omitted)
%	     'smart-crossings' - removes zero crossings which are highly likely noise
%
% Note:
%
%    The 'smart-crossings' assumes that the frequency of the process does not vary too
%    much (e.g. stays in bounds [freq/2, 2*freq], where freq is the mean frequency).
%    This assumption is necessary to be able to test for 'noisy' zero crossings.
%
%
% Returns:
%
%
%   ev_times : a vector of sample indices indicating the time of the events
%
% Examples:
%
%   t = 1:0.1:100;
%   x = sin(t);
%   p = mark_events(x, 'peaks');
%   plot(x, '-', p, ones(1,length(p)), '+')
%
%
function ev_times = mark_events(x, events)

    % PRE: set defaults if required
    if nargin < 2
        events = 'crossings';
    end

    % PRE: check inputs
    if( (size(x,1) > 1) && (size(x,2) > 1) )
        error('x must be a vector.');
    end

    N = length(x);

    if(strcmp(events, 'crossings'))

        % search for positive going zero crossings
        ev_times = find( (x(2:end) > 0) .* (x(1:end-1) < 0) );

    elseif(strcmp(events, 'peaks'))

        % search for positive going zero crossings to estimate period
        ev_times = find( (x(2:end) > 0) .* (x(1:end-1) < 0) );

        % compute 2/5 of mean points per period (assuming small no. of errors)
        min_above = floor(0.3 * mean(diff(ev_times))) + 1;

        % search for local maxima
        ev_times = find( (diff(x(1:end-1)) > 0) .* (diff(x(2:end)) < 0) ) + 1;

        % discard the local maxima which are too close to limits of series
        ev_times = ev_times( find((ev_times > min_above) .* (ev_times < N - min_above)));

        % discard all local maxima which are not maximal in their neighborhood
        for i=1:length(ev_times)
            ithp = ev_times(i);

            if max(x(ithp-min_above:ithp+min_above)) > x(ithp)
                ev_times(i) = 0;
            end
        end

        % remove discarded ev_times
        ev_times = nonzeros(ev_times);

    elseif(strcmp(events, 'smart-crossings'))

        % search for positive going zero crossings as above
        ev_times = find( (x(2:end) > 0) .* (x(1:end-1) < 0) );

        % compute 30% of mean points per period (assuming small no. of errors)
        min_above = floor(0.3 * mean(diff(ev_times))) + 1;

        % only accept the zero crossing if the signed area of points following
        % it is positive
        accepted = zeros(1,length(ev_times));
        for i=1:length(ev_times)-1
            if(sum(x((ev_times(i)+1):(min(ev_times(i)+min_above, N)))) > 0 ...
                && (ev_times(i+1) - ev_times(i)) > min_above/2)
                accepted(i) = 1;
            end
        end

        % filter out ev_times we are unsure about
        ev_times = ev_times(find(accepted));

    else

        error('Unknown event.');

    end

