%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: make_fourier_surr.m
%     Version: 1.0
%	 Date: 14 April 2008
%     License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    Constructs Fourier transform surrogates.  The function
%    works with raw time series.
%
% Arguments:
%
%    x : a matrix of time series for which surrogates should be created,
%        each time series is a column, matrix must have an even no. of rows
%
% Returns:
%
%    s : a matrix of the same size as the original, each column contains 
%        one surrogate
%
%
function s = make_fourier_surr(x)

	% retrieve the size
	[N, M] = size(x);

	% PRE1: length of time series is even
	if(mod(N, 2) == 1)
		error('Requires series of even length.')
	end

	% perform fft & rotate
	f = fft(x);
		
	% randomize the angles respecting the real series constraint
	half_len = N / 2 - 1;
	for j=1:M

		% compute a set of random rotations
		rotations = exp(1i * rand(half_len, 1) * 4*pi);
		fft_rot = [ 1; rotations; 1; flipud(conj(rotations)) ];

		% rotate the given column using the complex rotations
		f(:,j) = f(:,j) .* fft_rot;

	end

	% invert the fft transform
	s = real(ifft(f));

end
