%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: make_permutation_surr.m
%    Version: 1.0
%	 Date: 14 April 2008
%    License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    Constructs permutation surrogates by detecting the ends
%    of cycles and randomly changing the order of all the cycles 
%    except the first and last cycle.  The function works with phases.
%
% Arguments:
%
%    ph : a matrix of the phase time series for which surrogates should 
%         be created, each time series is a column
%
% Note:
%
%    Surrogates are constructed individually, i.e. each series is randomly
%    permuted by itself.
%
% Returns:
%
%    s : a matrix of the same size as the original, each column contains 
%        one surrogate
%
%
function s = make_permutation_surr(ph)

	% construct result set
	s = zeros(size(ph));

	% for each time series make a surrogate
 	for c=1:size(s,2)

		% find the cycle ends (-2*pi subtractions) in the phase
		rpos = find(diff(ph(:, c)) < -pi);

		% the number of permutable cycles is length(rpos) - 1
		% last cycle cannot be permuted as it is incomplete
		indices = randperm(length(rpos) - 1);

		% the surrogate has the same beginning segment as the source
		% series
		s(1:rpos(1)-1, c) = ph(1:rpos(1)-1, c);

		% now we start selecting cycles from the original
		% according to the random index set.
		pos = rpos(1);
		for i=1:length(indices)

		    % grab the index of the cycle we will now 'paste'
		    ii = indices(i);		

		    % compute the length of the cycle in samples
		    clen = rpos(ii+1)-rpos(ii);

		    % paste the sample
		    s(pos:pos+clen-1, c) = ph(rpos(ii):rpos(ii+1)-1, c);

		    % move the 'paste' position right behind the added cycle
		    pos = pos + clen;

		end

		% surrogate also has the same end
		s(pos:end, c) = ph(pos:end, c);

	end
    
end
