%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: compute_partition.m
%    Version: 1.0
%	 Date: 15 May 2011
%    License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%   Constructs an partition of the time series samples and returns it.
%   There are three choices for the computation of the partition.
%
%   The binning strategy can be selected
%   using a parameter.
%
%   NOTE: binning strategy must be one of:
%         'eqq' for equiquantal binning 
%         'eqdist' for equidistant binning
%         'eqq' for equiquantal
%
%   NOTE: for 'perm' binning, bin_count may only be one of [2, 6, 24, 120, 720].
%
% Synopsis:
%
%    p = compute_partition(ts, bin_count, strategy)
%
% Arguments:
%
%    ts : a matrix of input time series, each time series in one column.
%
%    bin_count : the number of bins into which the series should be distributed
%
%    strategy : partitioning strategy, one of 'eqq', 'eqdist', 'perm'
%
% Returns:
%
%    p: the mapping of samples to bin_count for each time series [1 .. bin_count]
%
%
function p = compute_partition(ts, bin_count, strategy)

    [M,N] = size(ts);

    % find bin indices according to required strategy
    if(strcmp(strategy, 'eqq'))
        p = eqq_partition(ts, bin_count);
    elseif(strcmp(strategy, 'eqdist'))
        min_ts = min(ts);
        bin_wdth = (max(ts) - min_ts) / bin_count;
        p = floor( (ts - repmat(min(ts), M, 1)) ./ repmat(bin_wdth, M, 1)) + 1;
        p(p > bin_count) = bin_count;
    elseif(strcmp(strategy, 'perm'))
        if(bin_count == 6)
            % there is an optimized permutation binning function for 6 bin_count
            p = perm_partition3_v2(ts);
        else
            perm_table = [ 2, 6, 24, 120, 720 ];
            pt_count = find(perm_table == bin_count) + 1;
            if(isempty(pt_count))
                error('Invalid number of bin_count selected in permutation binning, bin_count can be only [ 2, 6, 24, 120, 720 ].');
            end
            p = perm_partitionN(ts, pt_count);
        end
    else
        error('Unknown binning strategy, please use "eqq" or "eqdist" or "perm".');
    end
