%
%    This file is part of the INTERACT Toolbox
%    Copyright 2008 Martin Vejmelka
% 
%	 File: perm_partitionN.m
%    Version: 1.0
%	 Date: 18 April 2008
%    License: GPL version 3 or later
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%  
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%  
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    Author: Martin Vejmelka (bug reports: vejmelka at cs dot cas.cz)
%
% Description:
%
%    Constructs a permutation partition with a given number
%    of neighboring points used for comparison.  Uses sorting
%    internally and a special mapping to convert sorted indices
%    into bin assignments.  
%
% Synopsis:
%
%    p = perm_partitionN(x, npts)
%
% Arguments:
%
%    x: matrix of input time series, each time series is a column
%
%    npts : number of points to compare
%
% Note:
%
%    Due to the implementation, the number of neighboring points is
%    constrained to a maximum of 6 points.
%
% Returns:
%
%     p: the bins (1-6) computed from the assignment of the permutation method
%
%
function p = perm_partitionN(x, npts)

	% PRE: check parameters
	if(npts < 2 || npts > 6)
		error('Requested no. of neighbors must be between 2 and 6 inclusive.');
	end

	% ********** construct a mapping from sort index representation to bin number ********

	% construct a number system base with base = npts for
	num_base = [ones(1, npts) * npts] .^ [npts-1:-1:0];

	% the resulting bin_map
	bin_map = zeros(npts^npts, 1);

	% construct all possible permutations of bins and convert them to a numerical repr
	% in base 10: this will be non-contiguous
	bin_map_ndx = sum(perms([0:npts-1]) .* num_base(ones(factorial(npts),1), :), 2);

	% construct the mapping of the base 10 number to bin indices (order is arbitrary)
	bin_map(bin_map_ndx) = 1:factorial(npts);
	
	% ********** partition the given time series using the map ********

	% N - length of each series, M - number of series
	[N, M] = size(x);

	% partitions
	p = zeros(N - npts + 1, M);

	% for each time series
	for i=1:M
	
		% tmp buffer for sorting neighbors
		tmp = zeros(npts, N - npts + 1);
		for j=1:npts
			tmp(j,:) = x(j:N - npts + j, i)';
		end

		% retreive the sort indices
		[~, tmp_ndx] = sort(tmp);

		% compute the bins and assign
		p(:,i) = bin_map( sum((tmp_ndx - 1) .* num_base(ones(N - npts + 1, 1), :)' ) );

	end
